@@ -19,37 +19,61 @@
 import requests
 import json
 import inspect
+import simplejson
+import logging
 
-def __fname():
+logger = logging.getLogger('resultsdb_api')
+logger.addHandler(logging.NullHandler())
+
+
+def _fname():
     frame = inspect.currentframe().f_back
     name = frame.f_code.co_name
     return name
 
+
 class ResultsDBapiException(Exception):
-    pass
+    def __init__(self, message='', response=None):
+        ''':param response: :class:`requests.Response` object'''
+        self.message = message
+        self.response = response
+
+    def __str__(self):
+        return repr(self.message)
+
 
 class ResultsDBapi(object):
     def __init__(self, api_url):
         self.url = api_url
 
     def __raise_on_error(self, r):
-        if not r.ok:
-            try:
-                raise ResultsDBapiException(r.json()['message'])
-            except KeyError:
-                raise ResultsDBapiException('HTTP %s Error' % r.status_code)
+        if r.ok:
+            return
 
-    def create_job(self, ref_url, status = None, name = None):
-        data = dict(ref_url = unicode(ref_url), status = status, name = name)
+        try:
+            logger.warn('Received HTTP failure status code %s for request: %s',
+                        r.status_code, r.url)
+            raise ResultsDBapiException(
+                '%s (HTTP %s)' % (r.json()['message'], r.status_code), r)
+        except simplejson.JSONDecodeError as e:
+            logger.debug('Received invalid JSON data: %s\n%s', e, r.text)
+            raise ResultsDBapiException(
+                'Invalid JSON (HTTP %s): %s' % (r.status_code, e), r)
+        except KeyError:
+            raise ResultsDBapiException('HTTP %s Error' % r.status_code, r)
 
-        url = "%s/jobs"
+
+    def create_job(self, ref_url, status = None, name = None, uuid = None):
+        data = dict(ref_url = unicode(ref_url), status = status, name = name, uuid = uuid)
+
+        url = "%s/jobs" % self.url
         headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
         r = requests.post(url, data=json.dumps(data), headers = headers)
         self.__raise_on_error(r)
 
         return r.json()
 
-    def update_job(self, id = None, href = None, status = None):
+    def update_job(self, id = None, href = None, status = None, return_data = False):
         """
         id or href required, href has preference if both are set
         """
@@ -59,9 +83,9 @@ class ResultsDBapi(object):
         if href:
             url = href
         if not url:
-            raise TypeError("%s() requires either id or href parameter" % __fname())
+            raise TypeError("%s() requires either id or href parameter" % _fname())
 
-        data = dict(status = status)
+        data = dict(status = status, return_data = return_data)
 
         headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
         r = requests.put(url, data=json.dumps(data), headers = headers)
@@ -77,7 +101,7 @@ class ResultsDBapi(object):
         if href:
             url = href
         if not url:
-            raise TypeError("%s() requires either id or href parameter" % __fname())
+            raise TypeError("%s() requires either id or href parameter" % _fname())
 
         r = requests.get(url)
         self.__raise_on_error(r)
@@ -126,7 +150,7 @@ class ResultsDBapi(object):
         if href:
             url = href
         if not url:
-            raise TypeError("%s() requires either id or href parameter" % __fname())
+            raise TypeError("%s() requires either id or href parameter" % _fname())
 
         r = requests.get(url)
         self.__raise_on_error(r)
@@ -135,7 +159,6 @@ class ResultsDBapi(object):
 
     def get_results(self, page = None, limit = None, since = None, outcome = None, job_id = None, testcase_name = None, **kwargs):
         url = "%s/results" % self.url
-        dirty = False
         params = dict(page = page, limit = limit, since = since, outcome = outcome, job_id = job_id, testcase_name = testcase_name)
         params.update(kwargs)
         params = {key:value for key, value in params.iteritems() if value is not None}
